library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

-- provides a 1KHz clock

entity clock1k is
  port ( 
    clk : in STD_LOGIC;   -- full speed clock (100Mhz)
	 reset : in STD_LOGIC; -- reset signal
    clk1k : out STD_LOGIC -- 1KHz clock
  );
end clock1k;

architecture Behavioral of clock1k is

  -- set the constant number of ticks for 1KHz
  constant TOGGLE_TICKS : INTEGER := 50000; 

  signal currtoggle : STD_LOGIC := '0';
  signal nexttoggle : STD_LOGIC := '0';
  signal c : INTEGER range 0 to TOGGLE_TICKS := 0;

begin

  process (clk, reset)
  begin
    if reset = '1' then 
	   currtoggle <= '0';
		nexttoggle <= '0';
	   c <= 0;
	 else
      if clk = '1' and clk'Event then
		  if c = TOGGLE_TICKS then
		    c <= 0;
			 nexttoggle <= not currtoggle;
		  else
		    c <= c + 1;
			 currtoggle <= nexttoggle;
		  end if;
		end if;
	 end if;
  end process;
  
  clk1k <= currtoggle;
  
end Behavioral;

