library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Outputs data to the segment displays on the board.
-- The input is given by a unsigned byte per display
-- ('disp1', 'disp2;', 'disp3'). The display can be 
-- enabled/disabled using the 'enable' signal.

entity display is
  port ( 
    clk : in STD_LOGIC;                       -- main clock (100Mhz)
    reset : in STD_LOGIC;                     -- reset signal
	 enable : in STD_LOGIC;                    -- enable signal
    disp1 : in STD_LOGIC_VECTOR (7 downto 0); -- byte display 1
    disp2 : in STD_LOGIC_VECTOR (7 downto 0); -- byte display 2
    disp3 : in STD_LOGIC_VECTOR (7 downto 0); -- byte display 3
    seg : out STD_LOGIC_VECTOR (7 downto 0);  -- segment output
    adr : out STD_LOGIC_VECTOR (5 downto 0)   -- segment address
  );
end display;

architecture Behavioral of display is

 component control
  port ( 
    half_byte : in STD_LOGIC_VECTOR (3 downto 0);
    segment : in STD_LOGIC_VECTOR (2 downto 0);
    seg : out STD_LOGIC_VECTOR (7 downto 0);
    adr : out STD_LOGIC_VECTOR (5 downto 0)
  );
  end component;
  
  component iterator
  port (
    clk : in  STD_LOGIC;
	 reset : in STD_LOGIC;
    disp1 : in STD_LOGIC_VECTOR (7 downto 0);
    disp2 : in STD_LOGIC_VECTOR (7 downto 0);
    disp3 : in STD_LOGIC_VECTOR (7 downto 0);
    half_byte : out STD_LOGIC_VECTOR (3 downto 0);
    segment : out STD_LOGIC_VECTOR (2 downto 0)
  );
  end component;
  
  component clock1k
  port (
    clk : in STD_LOGIC;
	 reset : in STD_LOGIC;
    clk1k : out STD_LOGIC
  );
  end component;
  
  signal clk1k : STD_LOGIC;
  signal half_byte : STD_LOGIC_VECTOR (3 downto 0);
  signal segment : STD_LOGIC_VECTOR (2 downto 0);
  signal seg_t : STD_LOGIC_VECTOR (7 downto 0);
  signal adr_t : STD_LOGIC_VECTOR (5 downto 0);

begin

	control_impl: control
	port map (
	  half_byte => half_byte,
	  segment => segment,
	  seg => seg_t,
	  adr => adr_t
	);
	
	iterator_impl: iterator
	port map (
	  clk => clk1k,
     reset => reset,	  
	  disp1 => disp1,
	  disp2 => disp2,
	  disp3 => disp3,
	  half_byte => half_byte,
	  segment => segment
	);
	
	clock1k_impl: clock1k
	port map (
     clk => clk,
	  reset => reset,
     clk1k => clk1k
	);
	
	seg <= seg_t when enable = '1' and reset = '0' else "00000000";
	adr <= adr_t when enable = '1' and reset = '0' else "000000";
	
end Behavioral;

