library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

-- Smoothing the signal by filtering out small 
-- spikes of minimal lenght.

entity filter is
  port ( 
    clk : in STD_LOGIC;    -- main clock (100Mhz)
    reset : in STD_LOGIC;  -- reset signal 
    ir_in : in STD_LOGIC;  -- noisy signal
    ir_out : out STD_LOGIC -- filtered signal
  );
end filter;

architecture Behavioral of filter is

  -- Smoothing interval and noise detection
  constant hold_and_wait : INTEGER := 20;
  constant allowed_spikes : INTEGER := 3;

  signal c : INTEGER range 0 to hold_and_wait := 0;
  signal spikes : INTEGER range 0 to allowed_spikes := 0;
  signal smoothed : STD_LOGIC := '0';

begin

  process (clk, reset)
  begin
    if reset = '1' then
		spikes <= 0;
		smoothed <= '0';
	   c <= 0;
    else 
	   if clk = '1' and clk'Event then
		
		  -- output the smoothed value after 'hold_and_wait'
		  -- many steps
		  if c >= hold_and_wait then
		    ir_out <= smoothed;
		    smoothed <= ir_in;
		    spikes <= 0;
		    c <= 0;
	     else
		    c <= c + 1;
		  end if;
		  
		  -- swap the singal if there are two many differences
		  if ir_in /= smoothed then
		    if spikes = allowed_spikes then
		      smoothed <= ir_in;
				spikes <= 0;
				c <= 0;
			 else
			   spikes <= spikes + 1;
			 end if;
		  end if;
		end if;
	 end if;
  end process;
  
end Behavioral;

